/** MID2250 API.  API functions for the 2250 board.
 * Copyright 2006-2008 Sensoray Company Inc.
 * V117 notes:
 *    Temp files for MPEG1/MPEG2 removed.
 *
 *    Clarification about Pause/Resume. 
 *    SN_PauseRecord is not supported for MUX recording(Video plus Audio). 
 *    If you try to PauseRecord in this mode, the SDK will return an error:
 *    MID2250_ERR_INVALIDMODE
 *
 *    MID2250_REC types have been changed for better more reliable MPEG1 and MPEG2 recording
 *    with pause/resume.
 *    REC_VIDEOONLY will select the best video Only format for the encoding type
 *      Defaults to VES (the simplest, most flexible and widely accepted MPEG1/MPEG2 
 *      video only format) for MPEG1 and MPEG2.  MPEG4 video only(MP4S) requires a 
 *      Microsoft AVI container.
 *
 *   
 */
#ifndef _MID2250FUNC_H
#define _MID2250FUNC_H

#define MAX_DEVICES 30  // maximum number of 2250 in system

#include "mid2250types.h"

//////////////////////////////////////////////////////////////////////
// Begin Configuration and Setup functions                          

/** Opens the SDK/API.  Notes THIS resets all the settings to default. 
  * You must call SN_SetEncodeType and all the configuration routines after SN_Open is called
  * Default mode is MPEG4.
  * @return 0 on success
  */
MID2250_API int     SN_Open( void );

/** Cleans up and shuts down the API gracefully
 *  @return 0 on success
 */
MID2250_API int     SN_Close( void ); 

/** Sets the DLL mode of operation
 *  @param mode (0--normal(default), 1(MJPEG capture only. will set MJPEG encoding), 2 SampleGrabber Capture(see SG demo),
    3-DSHOW
 *  @return 0 on success
 */
MID2250_API int     SN_SetOpMode( MID2250_OPMODE mode, int board );

/** Sets the video input source. 
 *  @param input is the enumerated input MID2250_SOURCE enum(composite, svideo, DV) (see mid2250const.h)
 *  @return 0 on success
 */
MID2250_API int     SN_SetSource( MID2250_SOURCE input, int board);

/** Gets the current video source
 *  @param pSource pointer to returned value of current input source
 *  @return 0 on success 
*/
MID2250_API int     SN_GetSource( MID2250_SOURCE *pSource, int board );

/** set the current video system.  If board is currently streaming 
 *  board will be stopped and restarted.  If board recording, recording will
 *  be stopped.
 *  @param vidsys is the video system type
 *  @return 0 on success
 */
MID2250_API int		SN_SetVidSys( MID2250_VIDSYS vidsys, int board );

MID2250_API int     SN_GetVidSys( MID2250_VIDSYS *vidsys, int board);

/** 
 *  Sets the encoding type(see MID2250_ENCODING above).
 *  Calling this function will reset the IFrame only sequence mode if set.
 *
 *  @param encodeType type of encoding
 *  @return 0 on success, negative otherwise (see errors in mid2250const.h)
 */
MID2250_API int     SN_SetEncodeType( MID2250_ENCODING encodeType, int board);

/**
 *   Retrieves the current mpeg encoding
 *
 * @return current encoding setting. -1 on error
 */
MID2250_API int     SN_GetEncodeType( int board );

/** 
 *  Sets the bitrate.  Note: MAXIMUM Bitrate for half size is 2Mbit/s!
 *
 *  @param bitrate current bitrate
 *  @return 0 on success, negative otherwise (see errors in mid2250const.h)
 */
MID2250_API int     SN_SetBitrate( int bitrate, int board );

/**
 *   Retrieves the current bitrate
 *
 * @return current encoding setting. -1 on error
 */
MID2250_API int     SN_GetBitrate(int board );


/** retrieves current SDK and file information( see MIDSRAYSTATUS structure in types file)
 *  Use ONLY for SN_StartRecord, not for SN_StartRecordW
 *  @return 0 on success, negative otherwise (see errors in mid2250types.h)
 */
MID2250_API int		SN_GetStatus( MIDSRAYSTATUS *pStatus, int board );

/* Unicode supported version */
MID2250_API int		SN_GetStatusW( MIDSRAYSTATUS_UNICODE *pStatus, int board );

/////////////////////////////////////////////////////////
// video streaming functions


/** starts the streaming of video and audio
  * @param board is the board instance in the system( 0 for default or single board)
  * -1 "all board" feature removed.  use a loop in your user
  */
MID2250_API int     SN_StartStream(int board);

/** stops video/audio stream and stops recording if recording
 */
MID2250_API int     SN_StopStream( int board);

/** set the recording mode
 *  @param recMode MID2250_REC setting (see mid2250types.h)
 *  @return 0 on success
 */
MID2250_API int		SN_SetRecordMode( MID2250_REC recMode, int board);

/** get the recording mode
 *  @param recMode MID2250_REC setting (see mid2250types.h)
 *  @return 0 on success
 */
MID2250_API int		SN_GetRecordMode( MID2250_REC *recMode, int board);

/** 
 * Starts recording to file using filter graph previously set up.
 * @param filename is the filename to record to.  Use the full path, no extension
 * @param size is the size of the files to record in Megabytes
 *        if size full, then a new file started with an incremental number appended 
 *        to the filename. Set size to 0 for continuous recording or breakonrecsize = 0
 * @param breakonrecsize flag indicating whether to, 1=end recording when filesize reaches size, or 0=continue recording to new file.
 * @return 0 on success
 * Notes : 10/15/2007:  If using continuous record mode with clips, then a new file is searched and found each time.
 *                      If using single file record( breakonrecsize == 1), existing files will be overwritten.
 *                      Example:
 *                      If first recording to c:\\test.mpg.  Second record to c:\\test.mpg will overwrite first file
 *                             with breakonrecsize =1.
 *                      If first recording to c:\\test.mpg.  Second record to c:\\test.mpg will create new file c:\\test1.mpg
 *                             with breakonrecsize =0.
 * 
 */

MID2250_API int     SN_StartRecord(char *filename, int size, int breakonrecsize, int board );

MID2250_API int     SN_StartRecordW(LPWSTR filename, int size, int breakonrecsize, int board );

/** pause recording
 */
MID2250_API int		SN_PauseRecord	(int board );
/** resume recording
 */
MID2250_API int		SN_ResumeRecord	(int board );

/** stops recording to file. video/sound preview window will continue
 */
MID2250_API int		SN_StopRecord(int board );

/** returns estimated recording time left on disk(or -1 on error)
  * @param szDrive string pointing to current drive eg. "C:\"
  * @return time in seconds (or -1 on error)
  */
MID2250_API int     SN_GetTimeLeft(char *szDrive, int board);
MID2250_API int     SN_GetTimeLeftW(LPWSTR szDrive, int board);

/////////////////////////////////////////////////////////////
// Image capture or snapshot functions

/** takes a snapshot and saves to file in filename.  If MOTION_JPEG(MJPEG), 
  * this function will block until frame ready.
  * @param szFilename is the fully qualified file + path (without extension)
  * @param filetype is the file type to save( see mid2250const.h: MID2250_FILE_JPEG and/or MID2250_FILE_BMP)
  * @param qual-- jpeg quality, (use 25-100).
  * @return number of bytes read (image size) or -1 on error.
*/
MID2250_API int     SN_SnapshotToFile( const char *szFilename, int filetype, int qual , int board);
/* unicode version */
MID2250_API int     SN_SnapshotToFileW( const LPWSTR szFilename, int filetype, int qual , int board);



/* DLL compiled on win32.  
 *  If using VB, befine you BITMAPINFOHEADER function as follows with strong types.
     <StructLayout(LayoutKind.Sequential)> Public Structure BITMAPINFOHEADER
        Public biSize As Int32
        Public biWidth As Int32
        Public biHeight As Int32
        Public biPlanes As Int16
        Public biBitCount As Int16
        Public biCompression As Int32
        Public biSizeImage As Int32
        Public biXPelsPerMeter As Int32
        Public biYPelsPerMeter As Int32
        Public biClrUsed As Int32
        Public biClrImportant As Int32
    End Structure

 */

/** Please see examples in demo APP.
 * pFrame buffer needs to be larger than what is filled in for RGB.( 768* height(480 NTSC, 576 PAL) * 3)
 * SN_GetFrame retrieves a frame from and stores it in pFrame(previously assigned)
 *  @param inSize size of buffer [IN] supplied
 *  @param unsigned char *pFrame [IN] pointer to buffer
 *  @param BITMAPINFOHEADER *pBMI [OUT] pointer.  Currently, only the following parameters are filled in as below:
 *            pBMI->biHeight = mpginfo.coded_picture_height;
 *            pBMI->biWidth = mpginfo.coded_picture_width;
 *            pBMI->biSizeImage = bufSize; 
 *         
 *  @param type [IN] type of buffer requested (0 == YCbCr, 1 == RGB, 2 == Native)
 *     YCbCr buffer formatted as follows
 *     Y buffer 768*576 bytes
 *    Cb buffer next 768*576/4 bytes ( 4:2:2 sampling)
 *    Cr buffer next 768*576/4 bytes ( 4:2:2 sampling)
 *  RGB buffer 1 byte per component
 *  @param board [IN] board number 
 */
MID2250_API int SN_GetFrame( long inSize, unsigned char *pFrame, BITMAPINFOHEADER *pBMI, int type, int board);

// Playback functions                                                    
/**
 * filename:	[in] full path to the target file.
 *  rawType:    unused and reserved for future use. set as RAWTYPE_RGB24 or 1.
 *  vidnum:		[in] set to -1 always.  plays back to window.  Other values not supported.
 */
#define RAWTYPE_RGB24 1
MID2250_API int SN_SetPlayback(char *filename, int raw_type, int vidnum=-1);
/* set playback unicode*/
MID2250_API int SN_SetPlaybackW(LPWSTR filename, int raw_type, int vidnum=-1);

/** plays back the specified video clip in the current window
  * default is first
  * 11/12/2007.  Changed SN_PlaybackVideo.  Use SN_SetPlayback first
  */
MID2250_API int SN_PlaybackVideo( int vidnum=-1);

/** stops particular video playback
  * @return 0 on success 
  */
MID2250_API int SN_StopPlayback( int vidnum=-1);


/** pause/resume playback 
  * @param bPause bPause = TRUE(pause), bPause=FALSE(resume) 
  * @return 0 on success
  */
MID2250_API int		SN_PausePlayback( BOOL bPause, int vidnum=-1);

/** change playback rate 
 *  @param drate is a double specifying playback speed.  0.5 == half, 2.0 == double
 *  minimum speed is 0.5
 *  return 0 on success, -ve otherwise
 */
MID2250_API int		SN_PlaybackSetRate( double drate, int vidnum=-1 );

/** seek to a position relative to the percent of total stream.
 *  @param percent is an integer from 0(start) to range(end)
 *  @param range defines the percentage granularity
 *  @return 0 on success -1 otherwise
 */
MID2250_API int		SN_PlaybackSetSeekPosition( int percent, int range, int vidnum=-1);

/** 
 *  retrieves current position (percent of total stream) in the playback stream
 *  @param range defines the percentage granularity
 *  @return percent, an integer from 0(start) to range(end) or -1 on error
 */
MID2250_API int		SN_PlaybackGetSeekPosition( int range, int vidnum=-1 );

// General configuration options
/** set the brightness, contrast and saturation
  * @param param is the parameter to change( MID2250_LEVEL_CONTRAST, MID2250_LEVEL_BRIGHTNESS, MID2250_LEVEL_SATURATION, MID2250_LEVEL_HUE)
  * @param value is the value of the parameter
  *  ( CONTRAST, BRIGHTNESS, SATURATION: 0-100, default 50)
  *	 ( HUE: -50 to 50, default 0)
  * @return 0 on success
  */
MID2250_API int SN_SetLevels( int param, char value, int board );

/** get the brightness, contrast or saturation
  * @param param is the parameter to get( MID2250_LEVEL_CONTRAST, MID2250_LEVEL_BRIGHTNESS, MID2250_LEVEL_SATURATION, MID2250_LEVEL_HUE)
  * @param value is the returned value of the parameter
  *  ( CONTRAST, BRIGHTNESS, SATURATION: 0-100, default 50)
  *	 ( HUE: -50 to 50, default 0)
  * @return 0 on success
  */
MID2250_API int SN_GetLevel( int param, char *value, int board );


/** set audio input channel (default is LINE in)
  * @param audio is the audio channel input line-in, MID2250_AUDIO_LINE, MID2250_AUDIO_MIC
  * @param micBoost 0--normal, 1--+20dB
  * @return 0 on success
  */
MID2250_API int SN_SetAudioInput( MID2250_AUDIO_INPUT audio, int micBoost, int board);

/** get audio input channel (default is LINE in)
  * @param audio is the audio channel input line-in, MID2250_AUDIO_LINE, MID2250_AUDIO_MIC
  * @param micBoost 0--normal, 1--+20dB
  * @return 0 on success
  */
MID2250_API int SN_GetAudioInput( MID2250_AUDIO_INPUT *pAudio, int *pMicBoost, int board);



/** 
 *  Mutes the audio on the host computer.  Audio will still be recorded.  
 *  Default DLL mode is to have audio muted
 *  @param bMute whether to mute audio on PC
 *  @return 0 on success
 */
MID2250_API int		SN_SetMute( BOOL bMute, int board);

/** 
 * Gets the mute setting
 *
 * @param bMute pointer to retrieved mute setting
 * @return 0 on success 
 */
MID2250_API int		SN_GetMute( BOOL *bMute, int board);


/////////////////////////////////////////////////
// Caption(aka Overlay) functions
//NOTE: OSD color by default is white.
// All OSD overlay are currently the same color.
/** set the overlay position for overlay window
 * Overlay must be added after the stream is started
 * @param idx     Overlay number(index)
 * @param pos     position(left, top)
 * @param text    text to display( max 180 chars total for all overlays)
 * @param bEnable enable display region
 * @param bUpdate if setting multiple regions set to FALSE for all 
                     indices except the last.
 * @return 0 on success, -1 on failure, -2 if out of space
// NEW: 02/06/2007:  Overlay text allows changing font type. (See also end of file function to create OSD files)
//                   Also, supports 32x32 fonts (font index = 2 in overlay_text_t structure)
//                  FONT_INDEX = 2 uses TWO overlay positions
//                  IE.  if placing string "123" at idx = 0, internally, idx=1 will also be used
//                       because the 32x32 font requires 2 lines.(see notes below)
//  Example C code using MIXED fonts
//
//
// Also, note that OSDs should be organized in sequential Y order
// If Y is the same, enter the lower X ones first
// Also note that care should be taken to avoid overlap of the OSD.  Image blurring may occur
// if overlap occurs.
// 
// The MPEG standard works with macroblocks.  Macroblocks have a resolution of 16.
// Therefor X,Y positions 0-15 will be the same.  Coordinate entry is provided as a convenience instead
// of counting macroblocks.
// 32x32 font works with 4 macroblocks per character.  
// if -2 is returned unexpectantly, it may be because the large font.
{
    POINT pos;
    overlay_text_t ovltext;
    pos.x = 10;
    pos.y = 10;
    strcpy(ovltext.text,"\"2250\"");
    ovltext.fontidx = 2;
    SN_SetOverlayText( 0, &pos, &ovltext, TRUE, FALSE, m_iBoard);

    strcpy(ovltext.text,"Sensoray");
    pos.x = 100;
    pos.y = 200;
    ovltext.fontidx = 0;
    // NOTE: next overlay must be at position 0 + 2 because the position 0 one is 
    // a large font
    SN_SetOverlayText( 2, &pos, &ovltext, TRUE, TRUE, m_iBoard);
}
*/

MID2250_API int SN_SetOverlayText( int idx, const POINT *pPos, const overlay_text_t *pOvlText, BOOL bEnable, BOOL bUpdate, int board);


/** clears all overlay structures */
MID2250_API int SN_ClearOverlay(int board );

////////////////////////////////////////////
// miscellaneous functions (GPIO, Video display,
// callback routines, message notifications

/** Set the video size.  If stream is running, this call will stop and restart it
 *  @param iVidSize ( 0-- full size default, 1--half size(360x240 for NTSC),
                    ( 3--VGA size (Beta) )
 *  @return 0 on success
 */
MID2250_API int		SN_SetVidSize( int iVidSize, int board);

MID2250_API int     SN_GetVidSize( int *iVidSize, int board);

/** SetRenderVideo allows one to turn off the video window but still stream
 *  the video after SN_StartStream.  Video can also be recording with this setting off, but
 *  won't be displayed.
 *  @param bDisplayVideo whether to display video or not
 *  @return 0 on success
 */
MID2250_API int		SN_SetRenderVideo( BOOL bDisplayVideo, int board);
/** Gets the display video setting
 *  @param bDisplayVideo pointer to display video setting
 *  @return 0 on success
 */
MID2250_API int		SN_GetRenderVideo( BOOL *bDisplayVideo, int board);


/** set aspect ratio flag.
 *  Note: this is the DISPLAYed aspect ratio ONLY. 
 *  @param mode MID2250_ASPECT_NONE--stretched,  MID2250_ASPECT_CONST--maintain aspect ratio
 *  @return 0 on success, -ve on error
 */
MID2250_API int     SN_SetAspectRatio( MID2250_ASPECT_MODE mode, int board);

/** Get the aspect ratio mode
 *  @param mode returned aspect ratio mode
 *  @return 0 on success
 */
MID2250_API int		SN_GetAspectRatio( MID2250_ASPECT_MODE *mode, int board);



////////////////////////////////////////////////////////////////////////
// Callback functions from application


/** Repaint callback should be called when application receives a WM_PAINT event.  This is necessary to
 *  notify the video renderer of a repaint event. 
 *  @param hdc device context handle or NULL if unknown
 *  @return 0 on success
 */
MID2250_API int		SN_Repaint( HDC hdc);


/** Informs the video renderer that a WM_DISPLAYCHANGE message has been received by the application
 *  This callback should be called when the application has a WM_DISPLAYCHANGE event
 *  @return 0 on success
 */
MID2250_API int		SN_DisplayChange(int board);

/** Set the window for the video playback
 *  @param hwnd pointer to video display window
 *  @param reserved unused
 */
MID2250_API int		SN_SetPlaybackWindow( HWND hwnd, BOOL reserved );


/** Set the window for the video streaming per board( formerly UpdateClippingWindow)
 *  @param hwnd pointer to video display window
 *  @param BOOL reserved 
 *  @param board board number
 *  Note: this setting will override SN_SetRenderVideo and turn on 
 *  Video rendering(display) as it is assumed setting a window means display is wanted
 */
MID2250_API int		SN_SetBoardWindow( HWND hwnd, BOOL reserved, int board);

/** set remove msg
 *  @param messageHwnd window to notify when device removed
 *  @param removeMsg message on remove
 */

MID2250_API int SN_SetRemoveMsg( HWND messageHwnd, int removeMsg, int board);


/** Test whether device was removed or not
 * When SN_SetRemoveMessage is called, a message window handle and remove message is set.
 * When remove message is received by the application, this function should be called
 * to see if the device was removed.  DLL should be shut down if device terminated
 * @return 0 if not removed, 1 if removed
 */

MID2250_API int     SN_TestDeviceRemoval(int board);


/** set the video position in the clipping window 
 *  @param xpos x position of video
 *  @param ypos y position of video
 *  @param xsize size of video display
 *  @param ysize size of video display
 *  @param left_clip number of pixels at left of source video to clip
 *  @param top_clip y number of pixels at top of source video to clip
 *  @param right_clip number of pixels at right of source video to clip
 *  @param bottom_clip number of pixels at bottom of source video to clip
 *  @return 0 on success
 */ 
MID2250_API int     SN_SetVideoPosition( int xpos, int ypos, int xsize, int ysize, int left_clip, int top_clip, int right_clip, int bottom_clip, int board);

MID2250_API int     SN_GetNumBoards( int *pNumBoards);

/** Advanced setting for sequence mode(I,IP) frames.
 * 
 */
MID2250_API int     SN_SetMpegSequence( MID2250_SEQMODE seqMode, int board);
MID2250_API int     SN_GetMpegSequence( MID2250_SEQMODE *seqMode, int board);

/** returns the native video size */
MID2250_API int     SN_GetBoardVideoSize( RECT *rect, int board);
MID2250_API int     SN_GetPlaybackVideoSize( RECT *rect);

/** motion detection API. Beta */

/** sets or updates the motion detection region according to MID2250_MDConfig_t
 *  @param pMD see MID2250_MDConfig_t.  Warning, sensitivity 100 not recommended.
 *  @param event handle to event received on motion
 *  @param board board number( use 0 for default)
 *  @return 0 on success
 */
MID2250_API int     SN_SetMD( MID2250_MDConfig_t *pMD, HANDLE event, int board);
/** stops the MD interface.  Uploads NULL Motion Map to chip and turns off events
 */
MID2250_API int     SN_StopMD(int board);
/** Display MD Regions displays the regions in alpha blended form on the image.  Requires either XP or 
 *  a newer version of DirectX
 *  @param bOnOff whether to display the regions or not
 *  @param board board number
 */
MID2250_API int     SN_DisplayMDRegions( BOOL bOnOff, int board);
/** retrieves the Motion detection data.
 *  @param *region is the returned region mask.  region & 0x02: region 1.  region & 0x04: region 2, (region & 0x08) region 3
 *  the region mask should be used to detect if motion detected or not.
 *  @param *buf pointer to motion macroblock map.(must be size 208)
 *  @param bufsize size of bufmap.  Use 208.
 *  @param board
 *  The primary detection is by region.  If further information is wanted, the MB map will show if motion passed threshold in a
 *  given region on a macroblock by macroblock level(16 pixels x 16 pixels) on a bit level.
 *  Eg (buf[0] & 0x01) represents macroblock 1. (buf[0] &0x02) represents MB 2.  (buf[1] & 0x01) represents Macroblock 8
 */
MID2250_API int     SN_GetMDData( int *region, unsigned char *buf, int bufsize, int board);

// Enable Snapshots adds or removes the Sensoray Sample Grabber from the 
// filter graph.  If no snapshots are desired at all(compressed or decoded-uncompressed),
// consider disabled snapshots for lower CPU usage
MID2250_API int     SN_EnableSnapshots( BOOL bOnOff, int board);     

// if COMPRESSED MPEG frames are wanted(without decoding frames to YUV or RGB), leave
// SN_EnableSnapshots as default( TRUE), but call SN_SnapshotMPEGDecoding to 
// STOP decoding for snapshot purposes.  This will reduced CPU usage, but still allow
// snapshots with full API functionality.  If however, only MPEG snapshots are desired,
// consider using OpMode MID2250_OPMODE_MPEGONLY_CAPTURE.  See mpeg_capture demo for 
// details.  Use SN_GetFrame with type 2(native).
MID2250_API int     SN_SnapshotStreamDecoding( BOOL bOnOff, int board);

// Enables user to turn off AVSync.  If problems with sound card( or no sound card), 
// may need to disable AVSync(in driver) so frames are not dropped.
// NOTE: Recommended to set this before starting stream.
MID2250_API int     SN_AVSync( BOOL bOnOff, int board);

// CHANGES FFDSHOW Merit from preferred to Normal.
// Merit for FFDSHOW filter will be changed once SDK is open.  Change is permanent.
MID2250_API int     SN_ChangeFFMerit( int unused );

// starts streaming and record at same time.(So no preview is lost)
MID2250_API int SN_StartStreamAndRecord(char *filename, int size, int breakonrecsize, int board);

MID2250_API int SN_StartStreamAndRecordW(LPWSTR filename, int size, int breakonrecsize, int board);

// 10/19/2007.  Previously not recommended mode MPEG1 VCD has been removed.
// For MPEG2 mode only.  Set DVD Compliant flag.  Not the default in the SDK for backward compatibility.
// FOR MPEG2 sets DVD compliant flag.
MID2250_API int SN_SetDVDCompliant( BOOL bDVDCompliant, int board);

// advanced.  Stops the preview portion of the Graph.
MID2250_API int SN_StopPreview( int board );
MID2250_API int SN_StartPreview(int board);

// allows changing frame filter.  Will increase CPU usage greatly.  For MPEG1 and MPEG2 modes only
// MPEG4 only supports IFrame decoding.  MOTION_JPEG mode will always retrieve every hardware 
// compressed JPEG frame.
// filter == 0 all frames decoded (max cpu)
// filter == 1 IPFRAMES decoded
// filter == 2 IFRAMES decoded only( DEFAULT)
MID2250_API int     SN_SetFrameFilter( int filter, int board);


// advanced users.  Use with care
// v = vertical shift = +/- 3 lines
// h == horizontal shift = +/- 10 pixels
MID2250_API int SN_ShiftWin( int h, int v, int board);



/*
// for new OSD font files.  Updates the OSD files associated with the 2250 driver
// Note:  Will update driver font files.  If you create a bad font file, the defaults can be retrieved
// from the SDK driver directory (*.osd).
// font 0 size must be <= 16 . 16x16 font.
// font 1 size must be <= 16 . 16x16 font.
// font 2 size must be <= 32 . 32x32 font.
// idx == idx of font 0-2  (future use.  currently just uses fontname)
// osdfilename - filename of the osd file. Use this function to redistribute new font files
//               or call with full path 
//               FONT index 0 should be in WINDIR as Eng16_1.osd
//               FONT index 1 should be in %WINDIR% as Eng16_2.osd
//               FONT index 2(the 32x32 font) should be in %WINDIR% as Eng_hollow.osd
//       Eng16_1, Eng16_2 MUST be 16x16 fonts or less (fontsize 0-16)
//       Eng_hollow.osd MUST have a font size from fontsize 17-32
// New font files are loaded ONLY after SN_StartStream.
// tested font names are "Courier New", "Arial", "Times New Roman"
// iWeight use 400 for normal, 700 for bold, 800 for extrabold (see wingdi.h FW_BOLD, FW_NORMAL, etc).
//    recommend iWeight of 700 or FW_BOLD for default
// bItalic (italized font or not)

// Example C code calls:
{
    font_params_t param;
    memset( &param,0, sizeof(param));
    strcpy(param.fontname, "Times New Roman");
    param.iWeight = FW_EXTRABOLD;
    param.fontsize = 16;
    SN_CreateOSDFile( 0, "c:\\windows\\Eng16_1.osd", &param);
    strcpy(param.fontname, "Arial");
    SN_CreateOSDFile( 1, "c:\\windows\\Eng16_2.osd", &param);
    strcpy(param.fontname, "Times New Roman");
    param.fontsize = 32;
    param.iWeight = FW_EXTRABOLD;
    // Eng_hollow.osd is name of LARGE font(32x32) file.  name may be changed later.
    //"Hollow" fonts not are currently supported so the filename is misleading
    //If desired, please try the default hollow font.
    SN_CreateOSDFile( 2, "c:\\windows\\Eng_hollow.osd", &param);
 }
*/
typedef struct {
    int fontsize;
    int iWeight;
    int bItalic;
    int bHollow; // hollow font (see notes on color above for SN_SetOverlayText) 
    char fontname[100];
    int reserved[10]; // possible future use
} font_params_t;

typedef struct {
    int fontsize;
    int iWeight;
    int bItalic;
    int bHollow; // hollow font (see notes on color above for SN_SetOverlayText) 
    wchar_t fontname[100];
    int reserved[10]; // possible future use
} font_params_w_t;

MID2250_API int SN_CreateOSDFile( int idx, char *osdfilename, font_params_t *pParams);
MID2250_API int SN_CreateOSDFileW( int idx, LPWSTR osdfilename, font_params_w_t *pParams);

// Changes the frame rate (frames per second)
// Frame rate can only be changed before starting stream.
// 
// Use the default frame rate for normal operation.
// SetFPS to 0 for default 29.97NTSC, 25 PAL
// Allowed values depend on video system:
//  int ntsc_allowed[] = {5,10,15,30}; 
//  int pal_allowed[] = {5,12,25};
MID2250_API int SN_SetFPS( int fps, int board);

// Low Latency mode runs the direct show graph with sync clock = -1
// The video will go through the graph at the maximum speed without pacing
// The tradeoff is the audio/video may not be synched and the 
// video may not be completely smooth.
// Call this function before starting the stream if desired
MID2250_API int SN_LowLatency( BOOL bON, int board);



/** sets the audio input volume on a channel by channel basis.
 * @param channel = bitmask of input to set ( 1 -- left channel, 2-- right channel,  3--both)
 * @param level = volume control(0 to 31).  0= -34.5db, default=23=0dB, 31=max= +12 dB. (
 *                                          steps are 1.5dB
 * @param board = 2250 board
 */
#define AUD_CHANNEL_LEFT  1
#define AUD_CHANNEL_RIGHT 2
MID2250_API int SN_SetAudioInVol( int channel, int level, int board);

/** sets the audio input mute on a channel by channel basis.
 * @param channel = bitmask of input to set ( 1 -- left channel, 2-- right channel,  3--both, 4--microphone)
 * @param bMute = 1 mute (preserves volume settings), 0 unmuted
 * @param board = 2250 board
 */

//#define AUD_CHANNEL_LEFT  1
//#define AUD_CHANNEL_RIGHT 2
#define AUD_CHANNEL_MIC 4
MID2250_API int SN_SetAudioInMute( int channel, int bMute, int board);


// Sets the filtering in the preview graph.  Performance/Quality tradeoff
// Does not effect recording.
// filter = 0 default- none
// filter = 1 drop B frames 
// filter = 2 drop P frames
MID2250_API int SN_PreviewFilter( int filter, int board);

// Advanced.  Apart from the defaults, Sensoray has tested the following
// GOP size = 3 IPPIPPPIPP       seqmode IPONLY
// GOP size = 6 IPPPPPIPPPPPI    seqmode IPONLY (see SN_SetSeqMode)
// Invalid sizes/IPB combinations will cause recording AND/OR preview to not work
// Please know what you are doing before changing GOP size below
// use gopsize = -1 for default values
// calling this function clears the dvd compliant flag
MID2250_API int SN_SetGOPSize( int gopsize, int board);
MID2250_API int SN_GetGOPSize( int *gopsize, int board);

 
// Horizontal locking
// returns -1 or negative HRSEULT if call failed
// returns 0 if NOT locked
// returns 1 if LOCKED
MID2250_API int SN_GetHorizontalLock( int board);

/** Opens the SDK/API.  one board at a time
  * @return 0 on success
  */
MID2250_API int     SN_OpenBoard( int board );

/** Cleans up and shuts down the API gracefully
 *  One board at a time
 *  @return 0 on success
 */
MID2250_API int     SN_CloseBoard( int board ); 

MID2250_API int     SN_SetEmbeddedData( int insize, const void *data, int board);

MID2250_API int     SN_ClearEmbeddedData( int board);


#define MAX_LABEL_SIZE 10
typedef struct {
    char	labelText[MAX_LABEL_SIZE];
    float	startTime;
    int	displayMilliseconds;
    int	fontidx;
} overlay_timestamp_t;

MID2250_API int SN_SetTimestampOverlay( int idx, const POINT *pPos, const overlay_timestamp_t *pTimestamp, int board);
MID2250_API int SN_ClearTimestampOverlay(int board );
// For Compressed Capturing only USING callback functions
// Please see code_callback demo for examples of how this works
// This MODE is ONLY for capturing directly from the card
// In case NO preview is wanted, NO snapshots, NO recording, etc.
// It is just to get the MPEG data out of the card without any
// extra functionality. 
// Most Sensoray (SN) commands will NOT be available
// If you start the stream with this, don't expect to be able to record or preview.
// using the other functions.
// Mode = 0 (VES)
//
//  314 -> compressed stream -> samplegrabber -> null renderer
//
// Mode = 1  (PES)
//  2250-> compressed stream ----------->|-----|    |-------------|
//                                      | MUX | -> |SampleGrabber| -> Null Renderer
//      audio)->AudioEncoder----------->|-----|    |-------------|   
//
// Mode = 2  (PES no audio)
//  2250-> compressed stream ----------->|-----|    |-------------|
//                                      | MUX | -> |SampleGrabber| -> Null Renderer
//                                      |-----|    |-------------|   
//
//
// Mode == 3 ( VES + Audio encoded split streams( 2 callbacks))
//
//  2250-> compressed stream -> samplegrabber1 -> null renderer
//  2250--> audio stream------>encoder----------> samplegrabber2 -> null renderer
//
// Mode == 4 ( VES + Raw Audio streams( 2 callbacks))  
// 
//  2250-> compressed stream -> samplegrabber1 -> null renderer
//  2250--> audio stream------> samplegrabber2 -> null renderer
MID2250_API int SN_StartCBCaptureOnly( int mode, int board);
// stops Compressed capture only
MID2250_API int SN_StopCBCaptureOnly( int board);

// callback function
typedef int (*cbfunc_t)( BYTE *merged, long size, int board);

// FOR SN_StartCompressedCaptureOnly  filter graph ONLY or SN_StartCaptureOnly_CB
MID2250_API int SN_RegisterVidCB( cbfunc_t callback, int board);

// for SN_StartCompressedCaptureOnly with modes 3 or 4 ONLY!
MID2250_API int SN_RegisterAudCB( cbfunc_t callback, int board);















#endif
